@@ -52,7 +52,7 @@ static void
 epoll_poll (EV_P_ ev_tstamp timeout)
 {
   int i;
-  int eventcnt = epoll_wait (epoll_fd, epoll_events, epoll_eventmax, ceil (timeout * 1000.));
+  int eventcnt = epoll_wait (epoll_fd, epoll_events, epoll_eventmax, (int)ceil (timeout * 1000.));
 
   if (eventcnt < 0)
     {
@@ -75,7 +75,7 @@ epoll_poll (EV_P_ ev_tstamp timeout)
     {
       ev_free (epoll_events);
       epoll_eventmax = array_roundsize (epoll_events, epoll_eventmax << 1);
-      epoll_events = ev_malloc (sizeof (struct epoll_event) * epoll_eventmax);
+      epoll_events = (struct epoll_event *)ev_malloc (sizeof (struct epoll_event) * epoll_eventmax);
     }
 }
 
@@ -94,7 +94,7 @@ epoll_init (EV_P_ int flags)
   method_poll   = epoll_poll;
 
   epoll_eventmax = 64; /* intiial number of events receivable per poll */
-  epoll_events = ev_malloc (sizeof (struct epoll_event) * epoll_eventmax);
+  epoll_events = (struct epoll_event *)ev_malloc (sizeof (struct epoll_event) * epoll_eventmax);
 
   return EVMETHOD_EPOLL;
 }
